{
********************************************
        Assembly FunctionEngine Demo
********************************************
   (C) 2005 Parallax, Inc.            V1.0 
********************************************
}
CON
  _CLKMODE = XTAL1 + PLL16X
  _XINFREQ = 5_000_000

  _stack = ($3000 + $3000 + 100) >> 2                   'accommodate display memory and stack

  x_tiles = 16
  y_tiles = 12

  paramcount = 14       
  bitmap_base = $2000
  display_base = $5000

VAR
    long              tv_status                         '0/1/2 = off/visible/invisible                  read-only
    long              tv_enable                         '0/? = off/on                                   write-only
    long              tv_pins                           '%ppmmm = pins                                  write-only
    long              tv_mode                           '%ccinp = chroma,interlace,ntsc/pal,swap        write-only
    long              tv_screen                         'pointer to screen (words)                      write-only
    long              tv_colors                         'pointer to colors (longs)                      write-only
    long              tv_hc                             'horizontal cells                               write-only
    long              tv_vc                             'vertical cells                                 write-only
    long              tv_hx                             'horizontal cell expansion                      write-only
    long              tv_vx                             'vertical cell expansion                        write-only
    long              tv_ho                             'horizontal offset                              write-only
    long              tv_vo                             'vertical offset                                write-only
    long              tv_broadcast                      'broadcast frequency (Hz)                       write-only
    long              tv_auralcog                       'aural fm cog                                   write-only

    word              screen[x_tiles * y_tiles]
    long              colors[64]

OBJ
    tv    :       "tv"                                  'located in default Library
    gr    :       "graphics"                            'located in default Library
    AFn   :       "FunctionEngine"

PUB Setup | i,dx,dy

  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  'init colors
  repeat i from 0 to 63
    colors[i] := $00001010 * (5+4) & $F + $2B060C02

  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := display_base >> 6 + dy + dx * tv_vc + ((dy & $3F) << 10)

  'start and setup graphics
  gr.start                                   
  gr.setup(16, 12, 128, 96, bitmap_base)
    'start Assembly functions
    AFn.start                                           'Start the Assembly Function engine to operate in background

    MainLoop

PUB MainLoop|temp,RawDeg,Deg,Sine,ArcSine

    repeat
      gr.clear                                          'clear bitmap

      gr.colorwidth(2, 0)                               'Set color and pixel width
    
      gr.textmode(1,1,7,%0000)                          'Set text mode
      gr.text(-95,83,string("Assembly FunctionEngine Demo"))

'********************************************************************************************************************
      Deg := 4500                                        'Set DEG value (Human readable) multiplied by 100
          gr.text(-95,51,string("Sine      Deg ="))
          SimpleNum(-35,51,Deg,2)
      Deg := (Deg * 2048) / 9000                        'Convert DEG to 11-bit equivalent
      
      Sine := AFn.Sine(Deg)                             'Call Assembly routine to retrieve 16-bit Sine value

      Sine := (Sine * 10000) / 65535                    'Convert 16-bit value to Human readable format
          SimpleNum(50,51,Sine,4)
'********************************************************************************************************************
      ArcSine := 7071                                   'Set ARCSINE value (Human readable) multiplied by 10000
          gr.text(-95,27,string("ArcSine       =      Deg"))
          SimpleNum(-7,27,ArcSine,4)

      ArcSine := (ArcSine * 65535)/ 10000               'Convert ARCSINE to 16-bit equivalent

      Deg := AFn.ArcSine(ArcSine)                       'Call Assembly routine to retrieve 11-bit ArcSine value

      Deg := (Deg * 9000)/2048                           'Convert 11-bit value to Human readable format

          SimpleNum(40,27,Deg,2)

      gr.copy(display_base)                             'copy bitmap to display
      
PUB SimpleNum(x,y,DecimalNumber,DecimalPoint)|sign,DecimalIndex,TempNum,spacing,DecimalFlag,Digit
{     x,y           - lower right text coordinate
      DecimalNumber - signed Decimal number
      DecimalPoint  - number of places from the Right the decimal point should be
}
    spacing := 7
    DecimalIndex := 0

    TempNum := DecimalNumber                            'Preserve sign of DecimalNumber
    DecimalNumber := ||DecimalNumber
    if DecimalNumber <> TempNum 
       sign := 1
    else
       sign := 0

    repeat                                              'Print digits
      if DecimalIndex == DecimalPoint
         if DecimalPoint <> 0
            gr.text(x,y,@DP)                            'Insert decimal point at proper location
            x := x - spacing

      TempNum := DecimalNumber                          'Extract the least significant digit
      TempNum := DecimalNumber - ((TempNum / 10) * 10)

      Digit := $30 + TempNum                            'Display the least significant digit
      gr.text(x,y,@Digit)

      x := x - spacing
      DecimalIndex := DecimalIndex + 1
      DecimalNumber := DecimalNumber / 10               'Divide DecimalNumber by 10 

      if DecimalNumber == 0                             'Exit logic
         repeat while DecimalIndex < DecimalPoint       '   Do this if DecimalNumber is less than where the decimal point should be
            gr.text(x,y,@Zero)
            x := x - spacing
            DecimalIndex := DecimalIndex + 1
            DecimalFlag := 1
         if DecimalIndex == DecimalPoint                '   Set flag if DecimalNumber is equal to where the decimal point should be  
            DecimalFlag := 1   
         if DecimalFlag == 1
            gr.text(x,y,@DP)                            '   Insert decimal and leading Zero
            x := x - spacing
            gr.text(x,y,@Zero)                          
            x := x - spacing                
         if sign == 1                                   '   Restore sign of DecimalNumber
            gr.text(x,y,@Hyphen)
         quit
DAT

Zero                    word    $30
DP                      word    $2E
Hyphen                  word    $2D

tvparams                long    0               'status
                        long    1               'enable
                        'long  %011_0000        'pins Old Board
                        long    %001_0101        'pins New Board
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    60_000_000      '_xinfreq<<4 'broadcast
                        long    0               'auralcog                                            